﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Grayscale.P030_KifuReport.L0005_Report;
using System.Diagnostics;

namespace Grayscale.P030_KifuReport.L100_Report
{


    public class ReportArgsImpl : ReportArgs
    {
        /// <summary>
        /// 出力フォルダーへのパス。
        /// </summary>
        public string OutFolder { get { return this.outFolder; } }
        private string outFolder;

        /// <summary>
        /// 出力ファイルへのパス。
        /// </summary>
        public string OutFile { get { return this.outFile; } }
        private string outFile;

        /// <summary>
        /// 画像フォルダーへのパス。
        /// </summary>
        public string ImgFolder { get { return this.imgFolder; } }
        private string imgFolder;

        /// <summary>
        /// 駒画像ファイル。
        /// </summary>
        public string KmFile { get { return this.kmFile; } }
        private string kmFile;

        /// <summary>
        /// 数字画像ファイル。
        /// </summary>
        public string SjFile { get { return this.sjFile; } }
        private string sjFile;

        /// <summary>
        /// 駒の横幅
        /// </summary>
        public int KmW { get { return this.kmW; } }
        private int kmW;

        /// <summary>
        /// 駒の縦幅
        /// </summary>
        public int KmH { get { return this.kmH; } }
        private int kmH;

        /// <summary>
        /// 数字の横幅
        /// </summary>
        public int SjW { get { return this.sjW; } }
        private int sjW;

        /// <summary>
        /// 数字の縦幅
        /// </summary>
        public int SjH { get { return this.sjH; } }
        private int sjH;

        /// <summary>
        /// 将棋盤上の駒。[suji][dan]。
        /// sujiは1～9。danは1～9。0は空欄。つまり 100要素ある。
        /// 「K」「+p」といった形式で書く。(SFEN形式)
        /// </summary>
        public string[][] Ban { get { return this.ban; } }
        private string[][] ban;

        /// <summary>
        /// 先手の持ち駒の数。[0]から、飛,角,金,銀,桂,香,歩 の順。
        /// </summary>
        public int[] MotiP1 { get { return this.motiP1; } }
        private int[] motiP1;

        /// <summary>
        /// 後手の持ち駒の数。[0]から、飛,角,金,銀,桂,香,歩 の順。
        /// </summary>
        public int[] MotiP2 { get { return this.motiP2; } }
        private int[] motiP2;

        public ReportArgsImpl(string[][] ban, int[] motiP1, int[] motiP2,
            string outFolder,string outFile, string imgFolder, string kmFile, string sjFile,
            string kmW_str, string kmH_str, string sjW_str, string sjH_str)
        {
            // デバッグ
            {
                Debug.Assert(ban.Length == 10, "サイズ違反");
                Debug.Assert(ban[0].Length == 10, "サイズ違反");
                Debug.Assert(ban[1].Length == 10, "サイズ違反");
                Debug.Assert(ban[2].Length == 10, "サイズ違反");
                Debug.Assert(ban[3].Length == 10, "サイズ違反");
                Debug.Assert(ban[4].Length == 10, "サイズ違反");
                Debug.Assert(ban[5].Length == 10, "サイズ違反");
                Debug.Assert(ban[6].Length == 10, "サイズ違反");
                Debug.Assert(ban[7].Length == 10, "サイズ違反");
                Debug.Assert(ban[8].Length == 10, "サイズ違反");
                Debug.Assert(ban[9].Length == 10, "サイズ違反");
            }

            this.ban = ban;
            this.motiP1 = motiP1;
            this.motiP2 = motiP2;

            this.outFolder = outFolder;
            this.outFile = outFile;
            this.imgFolder = imgFolder;
            this.kmFile = kmFile;
            this.sjFile = sjFile;

            int kmW;
            if (!int.TryParse(kmW_str, out kmW))
            {
                kmW = 1;
            }
            this.kmW = kmW;

            int kmH;
            if (!int.TryParse(kmH_str, out kmH))
            {
                kmH = 1;
            }
            this.kmH = kmH;

            int sjW;
            if (!int.TryParse(sjW_str, out sjW))
            {
                sjW = 1;
            }
            this.sjW = sjW;

            int sjH;
            if (!int.TryParse(sjH_str, out sjH))
            {
                sjH = 1;
            }
            this.sjH = sjH;
        }
    }


}
