﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Grayscale.P030_KifuReport.L0005_Report;
using System.Drawing;

namespace Grayscale.P030_KifuReport.L100_Report
{


    public class ReporterImpl : Reporter
    {

        public const int MOTI_LEN = 7;//飛、角、金、銀、桂、香、歩の７つ。
        public const int BN_SUJIS = 9;//将棋盤は9筋。ban sujis
        public const int BN_DANS = 9;

        public const int BN_BRD_R_W = 1;//将棋盤の右辺の枠線幅。ban border right width。
        public const int BN_BRD_B_W = 1;


        /// <summary>
        /// 局面画像を描きだします。
        /// </summary>
        public void Write( ReportArgs args )
        {
            Bitmap bmp = new Bitmap(
                2*(args.KmW+2*args.SjW) + BN_SUJIS * args.KmW + BN_BRD_R_W,
                BN_DANS * args.KmH + BN_BRD_B_W
                );


            this.Paint(Graphics.FromImage(bmp),args);


            bmp.Save(args.OutFolder + args.OutFile);
        }


        /// <summary>
        /// 局面を描きます。
        /// </summary>
        public void Paint(Graphics g, ReportArgs args)
        {
            // 8×8 の将棋盤
            int bOx = args.KmW + 2 * args.SjW; // 将棋盤の左辺
            int bOy = 0;
            {

                // 縦線
                for (int col = 0; col < 10; col++)
                {
                    g.DrawLine(Pens.Black, col * args.KmW + bOx, 0 + bOy, col * args.KmW + bOx, 180 + bOy);
                }

                // 横線
                for (int row = 0; row < 10; row++)
                {
                    g.DrawLine(Pens.Black, 0 + bOx, row * args.KmH + bOy, 180 + bOx, row * args.KmH + bOy);
                }
            }

            // 盤上の駒
            {
                for (int suji = 1; suji < 10; suji++)
                {
                    for (int dan = 1; dan < 10; dan++)
                    {
                        string sign = args.Ban[suji][dan];
                        if ("" != sign)
                        {
                            Point pt = this.SignToXy(sign,args);
                            g.DrawImage(Image.FromFile(args.ImgFolder +args.KmFile),
                                new Rectangle((9 - suji) * args.KmW + bOx, (dan - 1) * args.KmH + bOy, args.KmW, args.KmH),//dst
                                new Rectangle(pt.X, pt.Y, args.KmW, args.KmH),//src
                                GraphicsUnit.Pixel
                                );
                        }
                    }
                }
            }

            // 後手の持駒 （飛,角,金,銀,桂,香,歩）
            {
                string[] signs = new string[] { "r", "b", "g", "s", "n", "l", "p" };
                int ox = 0;
                int oy = 0;
                for (int moti = 0; moti < 7; moti++)
                {
                    Point pt = this.SignToXy(signs[moti],args);
                    // 枚数
                    int maisu = args.MotiP2[moti];
                    if (0 < maisu)
                    {
                        //駒
                        g.DrawImage(
                            Image.FromFile(args.ImgFolder+args.KmFile),
                            new Rectangle(ox, (signs.Length- moti-1) * args.KmH + oy, args.KmW, args.KmH),//dst
                            new Rectangle(pt.X,pt.Y, args.KmW, args.KmH),//src
                            GraphicsUnit.Pixel
                            );

                        // 1桁目が先
                        {
                            int ichi = maisu % 10;
                            g.DrawImage(Image.FromFile(args.ImgFolder +args.SjFile),
                                new Rectangle(ox + args.KmW, (signs.Length- moti-1) * args.KmH + (args.KmH - args.SjH) + oy, args.SjW, args.SjH),//dst
                                new Rectangle(ichi * args.SjW, args.SjH, args.SjW, args.SjH),//src
                                GraphicsUnit.Pixel
                                );// 一の位
                        }

                        // 2桁目が後
                        {
                            int ju = maisu / 10;
                            if (ju < 1)
                            {
                                ju = -1;//空桁
                            }
                            g.DrawImage(
                                Image.FromFile(args.ImgFolder +args.SjFile),
                                new Rectangle(ox + args.KmW + args.SjW, (signs.Length - moti - 1) * args.KmH + (args.KmH - args.SjH) + oy, args.SjW, args.SjH),//dst
                                new Rectangle(ju * args.SjW, 0, args.SjW, args.SjH),//src
                                GraphicsUnit.Pixel
                                );// 十の位
                        }
                    }
                }
            }

            // 先手の持駒 （飛,角,金,銀,桂,香,歩）
            {
                string[] signs = new string[] { "R", "B", "G", "S", "N", "L", "P" };
                int ox = (args.KmW + 2 * args.SjW) + 9 * args.KmW + BN_BRD_R_W;
                int oy = (9 * args.KmW + BN_BRD_B_W) - 7*args.KmH;
                for (int moti = 0; moti < 7; moti++)
                {
                    Point pt = this.SignToXy(signs[moti],args);

                    // 枚数
                    int maisu = args.MotiP1[moti];
                    if (0 < maisu)
                    {
                        g.DrawImage(Image.FromFile(args.ImgFolder +args.KmFile),
                            new Rectangle(ox, moti * args.KmH + oy, args.KmW, args.KmH),//dst
                            new Rectangle(pt.X, pt.Y, args.KmW, args.KmH),//src
                            GraphicsUnit.Pixel
                            );//駒

                        // 十の位が先
                        {
                            int ju = maisu / 10;
                            if (ju < 1)
                            {
                                ju = -1;//空桁
                            }
                            g.DrawImage(Image.FromFile(args.ImgFolder +args.SjFile),
                                new Rectangle( ox + args.KmW, moti * args.KmH + (args.KmH - args.SjH) + oy, args.SjW, args.SjH),//dst
                                new Rectangle(ju * args.SjW, 0, args.SjW, args.SjH),//src
                                GraphicsUnit.Pixel
                                );// 十の位
                        }

                        // 一の位が後
                        {
                            int ichi = maisu % 10;
                            g.DrawImage(Image.FromFile(args.ImgFolder +args.SjFile),
                                new Rectangle( ox + args.KmW + args.SjW, moti * args.KmH + (args.KmH - args.SjH) + oy, args.SjW, args.SjH),//dst
                                new Rectangle(ichi * args.SjW, 0, args.SjW, args.SjH),//src
                                GraphicsUnit.Pixel
                                );// 一の位
                        }
                    }
                }
            }
        }


        private Point SignToXy(string sign, ReportArgs args)
        {
            Point pt;

            switch(sign)
            {
                case "P": pt = new Point(0 * args.KmW, 0 * args.KmH); break;
                case "p": pt = new Point(0 * args.KmW, 1 * args.KmH); break;
                case "L": pt = new Point(1 * args.KmW, 0 * args.KmH); break;
                case "l": pt = new Point(1 * args.KmW, 1 * args.KmH); break;
                case "N": pt = new Point(2 * args.KmW, 0 * args.KmH); break;
                case "n": pt = new Point(2 * args.KmW, 1 * args.KmH); break;
                case "S": pt = new Point(3 * args.KmW, 0 * args.KmH); break;
                case "s": pt = new Point(3 * args.KmW, 1 * args.KmH); break;
                case "G": pt = new Point(4 * args.KmW, 0 * args.KmH); break;
                case "g": pt = new Point(4 * args.KmW, 1 * args.KmH); break;
                case "K": pt = new Point(5 * args.KmW, 0 * args.KmH); break;
                case "k": pt = new Point(5 * args.KmW, 1 * args.KmH); break;
                case "R": pt = new Point(6 * args.KmW, 0 * args.KmH); break;
                case "r": pt = new Point(6 * args.KmW, 1 * args.KmH); break;
                case "B": pt = new Point(7 * args.KmW, 0 * args.KmH); break;
                case "b": pt = new Point(7 * args.KmW, 1 * args.KmH); break;
                case "+P": pt = new Point(8 * args.KmW, 0 * args.KmH); break;
                case "+p": pt = new Point(8 * args.KmW, 1 * args.KmH); break;
                case "+L": pt = new Point(9 * args.KmW, 0 * args.KmH); break;
                case "+l": pt = new Point(9 * args.KmW, 1 * args.KmH); break;
                case "+S": pt = new Point(10 * args.KmW, 0 * args.KmH); break;
                case "+s": pt = new Point(10 * args.KmW, 1 * args.KmH); break;
                case "+R": pt = new Point(11 * args.KmW, 0 * args.KmH); break;
                case "+r": pt = new Point(11 * args.KmW, 1 * args.KmH); break;
                case "+B": pt = new Point(12 * args.KmW, 0 * args.KmH); break;
                case "+b": pt = new Point(12 * args.KmW, 1 * args.KmH); break;
                default: pt = Point.Empty; break;
            }

            return pt;
        }
    }
}
