﻿using System;
using Grayscale.P030_KifuReport.L0005_Report;
using Grayscale.P006_Sfen.L100_Sfen;
using Grayscale.P006_Sfen.L0005_Sfen;
using System.Windows.Forms;
using System.Collections.Generic;
using System.IO;

namespace Grayscale.P030_KifuReport.L100_Report
{
    static class Program
    {

        static void AppendCommandline(Dictionary<string, string> dic)
        {
            string[] args = Environment.GetCommandLineArgs();

            foreach(string arg in args)
            {
                string name;
                string value;

                string rest = arg;
                if (!rest.StartsWith("--"))
                {
                    goto gt_Next1;
                }

                rest = rest.Substring(2);

                int eq = rest.IndexOf('=');
                if (-1 == eq)
                {
                    goto gt_Next1;
                }

                name = rest.Substring(0, eq).Trim();
                value = rest.Substring(eq + 1).Trim();

                if (value.StartsWith("\"") && value.EndsWith("\""))
                {
                    value = value.Substring(1, value.Length - 2);
                }

                if (dic.ContainsKey(name))
                {
                    dic[name] = value;
                }
                else
                {
                    dic.Add(name, value);
                }

            gt_Next1:
                ;
            }
        }


        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static void Main()
        {
            //
            // コマンドライン引数の例
            //
            // --position="position sfen 1nsgkgsnl/9/p2pppppp/9/9/9/P2PPPPPP/1B5R1/1NSGKGSNL w L2Pl2p 1 moves 5a6b 7g7f 3a3b" \
            // --outFolder="../../Logs/"
            // --outFile="_log_局面1.png"
            // --imgFolder="../../Data/img/gkLog/" \
            // --kmFile="koma1.png" \
            // --sjFile="suji1.png" \
            // --kmW=20 \
            // --kmH=20 \
            // --sjW=8 \
            // --sjH=12 \
            // --end
            //

            // ヌル防止のための初期値
            Dictionary<string, string> argsDic = new Dictionary<string, string>();
            argsDic.Add("position", "position startpos moves");
            argsDic.Add("outFolder", "./" );//出力フォルダー "../../Logs/"
            argsDic.Add("outFile", "1.png");//出力ファイル
            argsDic.Add("imgFolder", ".");//画像フォルダーへのパス image path
            argsDic.Add("kmFile", "2.png");//駒画像へのパス。
            argsDic.Add("kmW", "1");//駒の横幅。koma width
            argsDic.Add("kmH", "1");
            argsDic.Add("sjFile", "3.png");//数字・小
            argsDic.Add("sjW", "1");//数字の横幅。suji width
            argsDic.Add("sjH", "1");
            Program.AppendCommandline(argsDic);

            //foreach (KeyValuePair<string, string> entry in argsDic)
            //{
            //    MessageBox.Show("["+entry.Key + "]=[" + entry.Value+"]", "デバッグ");
            //}
            //MessageBox.Show("出力先=[" + Path.Combine(Application.StartupPath, argsDic["outPath"]) + "]", "デバッグ");
            

            Reporter reporter = new ReporterImpl();

            string[][] ban = new string[10][];// 将棋盤
            int[] motiP1 = new int[7];// 先手の持ち駒の数。[0]から、飛,角,金,銀,桂,香,歩 の順。
            int[] motiP2 = new int[7];
            // 全クリアー
            {
                // 将棋盤
                for (int suji = 0; suji < 10; suji++)
                {
                    ban[suji] = new string[10];

                    for (int dan = 0; dan < 10; dan++)
                    {
                        ban[suji][dan] = "";
                    }
                }
            }

            // デバッグ用
            {
                // SFEN を分解したい。
                //string sfen = "lnsgkgsn1/1r5b1/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSGKGSNL w - 1";
                //string sfen = "position sfen lnsgkgsnl/9/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSGKGSNL w - 1 moves 5a6b 7g7f 3a3b";
                //string sfen = "position sfen lnsgkgsnl/9/p1ppppppp/9/9/9/P1PPPPPPP/1B5R1/LNSGKGSNL w Pp 1 moves 5a6b 7g7f 3a3b";
                //string sfen = "position sfen 1nsgkgsnl/9/p2pppppp/9/9/9/P2PPPPPP/1B5R1/1NSGKGSNL w L2Pl2p 1 moves 5a6b 7g7f 3a3b";
                string sfen = argsDic["position"];

                RO_SfenStartpos ro_SfenStartpos;
                {
                    sfen = sfen.TrimStart();
                    if(sfen.StartsWith("position"))
                    {
                        sfen = sfen.Substring(8);
                    }

                    string rest;
                    if (!Util_SfenStartposReader.ReadString(sfen, out rest, out ro_SfenStartpos))
                    {

                    }
                }

                for (int suji = 1; suji < 10; suji++)
                {
                    for (int dan = 1; dan < 10; dan++)
                    {
                        ban[suji][dan] = ro_SfenStartpos.GetKomaAs(suji,dan);
                    }
                }

                motiP1[0] = ro_SfenStartpos.Moti1R;
                motiP1[1] = ro_SfenStartpos.Moti1B;
                motiP1[2] = ro_SfenStartpos.Moti1G;
                motiP1[3] = ro_SfenStartpos.Moti1S;
                motiP1[4] = ro_SfenStartpos.Moti1N;
                motiP1[5] = ro_SfenStartpos.Moti1L;
                motiP1[6] = ro_SfenStartpos.Moti1P;

                motiP2[0] = ro_SfenStartpos.Moti2r;
                motiP2[1] = ro_SfenStartpos.Moti2b;
                motiP2[2] = ro_SfenStartpos.Moti2g;
                motiP2[3] = ro_SfenStartpos.Moti2s;
                motiP2[4] = ro_SfenStartpos.Moti2n;
                motiP2[5] = ro_SfenStartpos.Moti2l;
                motiP2[6] = ro_SfenStartpos.Moti2p;
            }

            reporter.Write(new ReportArgsImpl(
                ban, motiP1, motiP2,
                argsDic["outFolder"],
                argsDic["outFile"],
                argsDic["imgFolder"],
                argsDic["kmFile"],
                argsDic["sjFile"],
                argsDic["kmW"],
                argsDic["kmH"],
                argsDic["sjW"],
                argsDic["sjH"]
                ));
        }
    }
}
